from uvicore.typing import Dict

class Configurator(Dict):
    """Ohmyi3 Base Configurator"""

    def byhost(self, attribute, values = None):
        """Override attribute values for specific hosts"""
        if type(attribute) == dict:
            # Override any number of values using a large Dictionary
            # Example:
            # self.byhost({
            #     'p15': {
            #         'terminal': 'xfce4-terminal',
            #         'calculator': 'gcalc'
            #     }
            # })
            for host, attributes in attribute.items():
                if self.hostname == host:
                    for attr, value in attributes.items():
                        self.dotset(attr, value)
        else:
            # Override a single value for a number of hosts
            # Example:
            # self.byhost('theme', {
            #     'sunjaro': 'manjaro',
            #     'p53': 'amber',
            # })
            for host, value in values.items():
                if self.hostname == host:
                    #setattr(self, attribute, value)
                    self.dotset(attribute, value)




import os
import uvicore
from ohmyi3 import util
from ohmyi3.util import path, plugin
from uvicore.typing import Dict
from ohmyi3.app import Application
from uvicore.support.dumper import dump, dd
from ohmyi3.configurator import Configurator

# Welcome to ohmyi3! A better way to dynamically control i3 configs and themes!
# See https://github.com/ohmyi3/ohmyi3

# This is the main control and configuratoin script for ohmyi3!
# All self.* variables are accessible as jinja2 variables to dynamically control
# your custom config.d/* i3 config files and themes.  Themes and plugins
# are provided by the Community (that means you!).  Plugins can be run from
# the even hooks (after_generate) to configure and tweak any part of your system
# like setting dmenu, archey, conky, nitrogen....all from the selected theme!
# Enjoy, and please contribute!

class Ohmyi3(Configurator):
    def __init__(self, app: Application):

        # Application instance
        self.app = app

        # Hostname and user from system
        self.hostname = util.hostname()
        self.user = util.loggedinuser()

        # Has battery (is laptop)
        self.has_battery = True
        self.byhost('has_battery', {
            'sunjaro': False,
            'p53': True,
            'p15': True,
            'p14s': True,
        })

        # Desktop Environment (kde, xfce, i3, cinnamon, mate, gnome)
        # I like to run i3 inside kde and xfce etc...  If runing under these DE's
        # I need to tweak the configs (no screen lock, different autostarts etc...)
        self.desktop = 'i3'
        self.byhost('desktop', {
            'sunjaro': 'kde',
            'p53': 'i3',
            'p15': 'i3',
            'p14s': 'cinnamon',
        })

        # Theme (amber, archlinux, manjaro, pink)
        self.theme = 'manjaro'
        self.byhost('theme', {
            'sunjaro': 'manjaro',
            'p53': 'manjaro',
            'p15': 'manjaro',
            'p14s': 'pink',
        })

        # Custom wallpaper base folder
        self.wallpaper_base = '~/Wallpaper'
        self.byhost('wallpaper_base', {
            'sunjaro': '~/Pictures/Wallpaper',
            'p53': '~/Pictures/Wallpaper',
        })

        # Other details for each theme
        # Wallpaper is an OVERRIDE, else defaults to the themes folder/background.[jpg|png]
        self.themes = Dict({
            'amber': {
                #'wallpaper': 'Abstract/cracked_orange.jpg',
                #'wallpaper': 'Manjaro/antelope-canyon-984055.jpg',
                #'wallpaper': 'Scenes/digital_sunset.jpg',
                #'wallpaper': 'LinuxMint/linuxmint-vera/mpiwnicki_red_dusk.jpg',
                #'wallpaper': 'LinuxMint/linuxmint-vanessa/navi_india.jpg',
                'wallpaper': 'LinuxMint/linuxmint-una/nwatson_eclipse.jpg',
                #'wallpaper': 'LinuxMint/linuxmint-vera/navi_india.jpg',
                #'wallpaper': 'LinuxMint/linuxmint-ulyssa/tangerine_nanpu.jpg',
                #'wallpaper': 'Manjaro/sky-3189347.jpg',
                'archey3': 'yellow',
            },
            'archlinux': {
                #'wallpaper': 'Archlinux/349880.jpg',
                'archey3': 'blue',
            },
            'manjaro': {
                #'wallpaper': 'Manjaro/illyria-default-lockscreen-nobrand.jpg',
                #'wallpaper': 'Manjaro/wpM_orbit2_textured.jpg'
                'archey3': 'green',
            },
            'pink': {
                #'wallpaper': 'Abstract/artistic_colors2.jpg',
                #'wallpaper': 'Landscape/backlit-chiemsee-dawn-1363876.jpg',
                #`'wallpaper': 'Manjaro/DigitalMilkyway.png',
                #'wallpaper': 'LinuxMint/linuxmint-una/eeselioglu_istanbul.jpg',
                #'wallpaper': 'Abstract/neon_huawei.jpg',
                'archey3': 'cyan',
            },
        })
        self.byhost({
            'p15': {'themes.manjaro.wallpaper': 'Manjaro/wpM_orbit2_textured.jpg'}
        })
        # self.byhost('themes.amber.wallpaper', {
        #     'p15': None,
        # })

        # Default font for window titles (not bar.font)
        self.font = 'xft:URWGothic-Book 9'

        # i3bar configs
        # All themes may obey these global bar configs, or they may set their own
        self.bar = Dict({
            #'cmd': 'i3bar',
            'cmd': 'i3bar --transparency',
            'status_cmd': 'i3status',
            'position': 'bottom',
            'font': 'xft:URWGothic-Book 8',

            # Hide or show the bar
            'mode': 'dock', # dock|hide|invisible
            'hidden_state': 'show', # hide|show

            # Modifier makes the hidden bar show up while key is pressed
            'modifier': 'none',
            #'modifier': 'Ctrl+$alt',
        })
        # If not using pure i3 (i3 in kde or xfce... hide the bar)
        if self.desktop != 'i3':
            self.bar.mode = 'hide'
            self.bar.hidden_state = 'hide'
        # self.byhost('bar.position', {
        #     'p15': 'top'
        # })
        # self.byhost({
        #     'p15': {
        #         'bar.cmd': 'xxx',
        #         'bar.position': 'yyy'
        #     }
        # })

        # Preferred applications, could change depending on DE installed
        self.terminal = 'konsole'
        self.calculator = 'kcalc'
        self.screenshot = 'spectacle' # i3-scrot
        self.dmenu = path('~/.files/scripts/dmenu-run-blue')
        self.volumectrl = 'pavucontrol' # konsole -e 'alsamixer'
        if self.theme == 'manjaro':
            self.dmenu = path('~/.files/scripts/dmenu-run-green')
        # self.byhost({
        #     'p15': {
        #         'terminal': 'xfce4-terminal',
        #         'calculator': 'gcalc'
        #     }
        # })

        # Dynamically Load and Instantiate Plugins
        self.plugins = Dict({
            'nitrogen': plugin('nitrogen.Nitrogen')(self, 'tt'),
            'archey3': plugin('archey3.Archey3')(self)
        })


    async def before_generate(self):
        """This hook fires before the new i3 config is generated"""


    async def after_generate(self):
        """This hook fires after the new i3 config is generated"""

        # Set themed wallpaper
        self.plugins.nitrogen.set_wallpaper()

        # Set themed archey in my .zshrc and or .bashrc
        self.plugins.archey3.set_archey()




import os
import platform
from datetime import datetime
from uvicore.support.module import load
from jinja2 import Environment as JinjaEnv
from uvicore.support.dumper import dump, dd
from jinja2 import FileSystemLoader as JinjaLoader

def plugin(module, **kwargs):
    return load('plugins.' + module).object


def path(location, must_exist=False, notfound_message=None):
    """Path helper with realpath and expanduser capabilities"""
    if type(location) == list:
        # If location is a list, join together with /
        location = "/".join(location)

    # Get real expanded path
    real = os.path.realpath(os.path.expanduser(location))

    # If must_exist, ensure it exists or display a message and exit
    if must_exist:
        if not os.path.exists(real):
            print(f"{real} not found")
            if notfound_message: print(notfound_message)
            exit(1)

    # Return real expanded path
    return real

def exists(location):
    """Check if path exists, using path() helper for realpath and expanduser"""
    return os.path.exists(path(location))


def now(dateformat='%Y-%m-%d_%H-%M-%S'):
    """Get now() date time in specified format (defaults 2021-01-24_19-24-58)"""
    return datetime.now().strftime(dateformat)


def shell(cmd):
    return os.system(cmd)


def template(path, file, **kwargs):
    """Render a file through the Jinja2 templating system"""
    loader = JinjaLoader(searchpath=path)
    env = JinjaEnv(loader=loader)
    template = env.get_template(file)
    rendered = template.render(kwargs)
    return rendered

def loggedinuser():
    return os.getlogin();

def hostname():
    return platform.node()



import uvicore
from ohmyi3.app import Application
from uvicore.support.dumper import dump, dd
from uvicore.exceptions import SmartException
from uvicore.console import command, argument, option

@command()
async def cli():
    """Display ohmyi3 configuration information"""

    # Get the application instance
    app = Application();

    uvicore.log.header("Ohmyi3 User Specific Configurator Instance")
    del app.configurator.app
    dump(app.configurator)

    uvicore.log.nl().nl()
    uvicore.log.header("Ohmyi3 Application Instance")
    del app.configurator
    dump(app)




import os
import sys
import shutil
import uvicore
from uvicore.typing import Dict
from uvicore.support.dumper import dump, dd
from ohmyi3.util import path

class Application(Dict):

    def __init__(self, must_exist = True):

        # Ohmyi3 main folder (~/.config/ohmyi3)
        self.config_folder = path(uvicore.config('ohmyi3.config_path'), must_exist,
            notfound_message="Perhpas you havent run 'i3ctl init' yet??")

        # Ohmyi3 i3 configs folder (~/.config/ohmyi3/configs)
        self.configd_folder = path([self.config_folder, 'config.d'], must_exist)

        # Ohmyi3 i3 themes folder (~/.config/ohmyi3/themes)
        self.themes_folder = path([self.config_folder, 'themes'])

        # i3 config folder (~/.config/i3)
        self.i3config_folder = path(uvicore.config('ohmyi3.i3config_path'))

        # i3status path (~/.config/i3status)
        self.i3status_folder = path(uvicore.config('ohmyi3.i3status_path'))

        # Stubs folder
        self.app_folder = path(uvicore.app.package(main=True).path, True)
        self.stubs_folder = path([self.app_folder, 'stubs'], True)

        # Import user settings (~/.config/ohmyi3/settings.py)
        if must_exist:
            self.configurator = self._import_configurator();


    def _cleanup(self):
        pycache = path([self.config_folder, '__pycache__'])
        if os.path.exists(pycache):
            shutil.rmtree(pycache)


    def _import_configurator(self):
        """Dynamically import the users ~/.config/ohmyi3/configurator.py"""
        sys.path.append(os.path.realpath(self.config_folder))
        from configurator import Ohmyi3
        return Ohmyi3(self)


